[Setting category="Medals" name="Show Author"]
bool showAuthor = true;

[Setting category="Medals" name="Show Gold"]
bool showGold = false;

[Setting category="Medals" name="Show Silver"]
bool showSilver = false;

[Setting category="Medals" name="Show Bronze"]
bool showBronze = false;

[Setting category="Medals" name="Shadows" description="Couldn't decide if it looks better with or without shadows, so it's up to you."]
bool shadows = true;

//rendering vars
nvg::Font nvgFont;
nvg::Texture@ nvgAuthor;
nvg::Texture@ nvgGold;
nvg::Texture@ nvgSilver;
nvg::Texture@ nvgBronze;

const vec2 coords = vec2(1580, 725);

const int fontSize = 35;
const float textHeight = 0.72*fontSize;
float textWidth = 0;
const float skewAngle = -.21;

const float medalSize = 32.0;
const float medalPadding = 3.0;
const float spacing = Math::Max(textHeight,medalSize)*1.1;

//states of ML pages refresh
bool campaignRefreshNeeded = true;
bool totdRefreshNeeded = true;

//patch data
const string patchedDAName = "patchedcustommedalinfo";
const int nbInfos = 5; //the number of infos the ML pages give us


void Main() {
	//we start by patching the ML layers
    auto app = cast<CTrackMania@>(GetApp());
	while(app.MenuManager is null 
		|| app.MenuManager.MenuCustom_CurrentManiaApp is null 
		|| app.MenuManager.MenuCustom_CurrentManiaApp.UILayers.Length <40)
		yield();
	
    auto uiLayers = app.MenuManager.MenuCustom_CurrentManiaApp.UILayers;
	for (uint i = 0; i < uiLayers.Length; ++i) {
		CGameUILayer@ uiLayer = uiLayers[i];
		string header = uiLayer.ManialinkPageUtf8.SubStr(0, 100);

		if (header.IndexOf("\"Page_CampaignDisplay\"") != -1){
			if(PatchCampaignLayer(uiLayer) == -1)
				print("Campaign patch failed!");
		}
		
		if (header.IndexOf("\"Page_MonthlyCampaignDisplay\"") != -1){
			if(PatchTotdLayer(uiLayer) == -1)
				print("Totd patch failed!");
		}
	}
	
	//Font and Medals loading
	nvgFont = nvg::LoadFont("DroidSans-Bold.ttf");
    @nvgAuthor = nvg::LoadTexture("medal_author_32.png", nvg::TextureFlags::GenerateMipmaps);
    @nvgGold = nvg::LoadTexture("medal_gold_32.png", nvg::TextureFlags::GenerateMipmaps);
    @nvgSilver = nvg::LoadTexture("medal_silver_32.png", nvg::TextureFlags::GenerateMipmaps);
    @nvgBronze = nvg::LoadTexture("medal_bronze_32.png", nvg::TextureFlags::GenerateMipmaps);
	
	//text width
	nvg::FontFace(nvgFont);
	nvg::FontSize(fontSize);
	textWidth = nvg::TextBounds("0:00.000").x;
}

//returns index of patch if success, -1 otherwise
int PatchLayer(CGameUILayer@ uiLayer, const string &in mapInfoName){
	//we check that the layer is not already patched
	if(uiLayer.ManialinkPageUtf8.IndexOf(patchedDAName) != -1)
		return -1;
	
	//we patch the ML page to add our custom info as a data attribute of the frame_maprankings
	int index = uiLayer.ManialinkPageUtf8.IndexOf("TMNext_MapRankings::UpdateMapRanking");
	if(index == -1)
		return -1;
	uiLayer.ManialinkPageUtf8 = 
		uiLayer.ManialinkPageUtf8.SubStr(0, index) 
		+ "_State.Controls.Frame_MapRankings.DataAttributeSet(\""+patchedDAName+
		"\",\"\"^"+mapInfoName+".Uid^\",\"^"+mapInfoName+".AuthorTime^\",\"^"+
		mapInfoName+".GoldTime^\",\"^"+mapInfoName+".SilverTime^\",\"^"+mapInfoName+".BronzeTime);\n"
		+ uiLayer.ManialinkPageUtf8.SubStr(index,uiLayer.ManialinkPageUtf8.Length-index);
	return index;
}

int PatchTotdLayer(CGameUILayer@ uiLayer) {
	return PatchLayer(uiLayer,"_DailyMap.Map");
}

int PatchCampaignLayer(CGameUILayer@ uiLayer) {
	return PatchLayer(uiLayer,"MapData");
}

//Triggers refresh of the ML page. If we don't do this, the page only loads properly on the second render ILOVEMANIALINKPAGES
void RefreshPage(CTrackMania@ app, CGameUILayer@ uiLayer){
	MwFastBuffer<wstring> Data;
	Data.Add("Router_EnteringRoute");
	app.MenuManager.MenuCustom_CurrentManiaApp.LayerCustomEvent(@uiLayer,"Router_AddPendingEvent",Data);
}

void Render() {
	//most of the UI checks here come from NaNInf's ChampionMedals
    auto app = cast<CTrackMania@>(GetApp());
	if (app.Network.ClientManiaAppPlayground !is null //we are playing
        || app.MenuManager.MenuCustom_CurrentManiaApp is null //probably never null
		|| (app.LoadProgress !is null && app.LoadProgress.State != NGameLoadProgress_SMgr::EState::Disabled)) //loading in progress
		return;
	
	//we first check if Campaign or Totd page is visible
    auto uiLayers = app.MenuManager.MenuCustom_CurrentManiaApp.UILayers;
	CGameUILayer@ mainUILayer;
	for (uint i = 0; i < uiLayers.Length; ++i) {
		CGameUILayer@ uiLayer = uiLayers[i];
		if(!uiLayer.IsVisible) 
			continue;

		string header = uiLayer.ManialinkPageUtf8.SubStr(0, 100);
		if (header.IndexOf("\"Page_CampaignDisplay\"") != -1){
			@mainUILayer = uiLayer;
			if(campaignRefreshNeeded) //ML page needs refresh upon first display
				RefreshPage(app,uiLayer);
				campaignRefreshNeeded = false;
			break;
		}
		if (header.IndexOf("\"Page_MonthlyCampaignDisplay\"") != -1){
			@mainUILayer = uiLayer;
			if(totdRefreshNeeded) //ML page needs refresh upon first display
				RefreshPage(app,uiLayer);
				totdRefreshNeeded = false;
			break;
		}
	}
	if(mainUILayer is null) 
		return;
	
	//we check if the frame with player's time is visible (hidden if we're in records tab, etc...)
	auto medalFrame = cast<CGameManialinkFrame@>(mainUILayer.LocalPage.GetFirstChild("tmnext-map-rankings_frame-medal"));
	if(!medalFrame.Visible) 
		return;
	
	//we get the frame where the ML page stored our info
	auto mapRankingsFrame = cast<CGameManialinkFrame@>(mainUILayer.LocalPage.GetFirstChild("frame-map-rankings"));
	auto infos = mapRankingsFrame.DataAttributeGet(patchedDAName).Split(',');
	
	if(infos.Length == nbInfos && infos[0] !="") {
		//if no time is set, we draw a translucid background behind the medals for better reading
		bool noTime = cast<CGameManialinkLabel@>(medalFrame.GetFirstChild("tmnext-map-rankings_label-player-time")).Value == "";
		int mi = 0;
		if(showAuthor)
			DrawMedal(FormatTime(infos[1]), nvgAuthor, mi++, noTime);
		if(showGold)
			DrawMedal(FormatTime(infos[2]), nvgGold, mi++, noTime);
		if(showSilver)
			DrawMedal(FormatTime(infos[3]), nvgSilver, mi++, noTime);
		if(showBronze)
			DrawMedal(FormatTime(infos[4]), nvgBronze, mi++, noTime);
	}
}

string FormatTime(const string &in timeString){
	return Time::Format(Text::ParseInt(timeString));
}

//this part is inspired by Miss' LoadProgress and NaNInf's ChampionMedals
void DrawMedal(const string &in str, nvg::Texture@ medal, int medalIndex = 0, bool drawBackground = false){
	vec2 totalBounds = vec2(medalSize + medalPadding + textWidth, Math::Max(medalSize, textHeight));
	nvg::Translate(coords + vec2(-5*medalIndex,medalIndex*spacing) -0.5*totalBounds);
	
	//draw shadowed background for better reading
	if(drawBackground){
		nvg::BeginPath();
		vec2 bgBounds = totalBounds + vec2(nvg::TextBounds(str).x - textWidth, 0);
		nvg::FillPaint(nvg::BoxGradient(.1*bgBounds, .9*bgBounds, 5.0, 30.0,  vec4(0, 0, 0, .7), vec4(0, 0, 0, 0)));
		nvg::Fill();
		nvg::ClosePath();
	}

	//draw medal
	vec2 medalPos = vec2(0,totalBounds.y/2.0-medalSize/2.0);
	vec2 medalBounds = vec2(medalSize,medalSize);
	
	if(shadows){
		nvg::BeginPath();
		nvg::FillPaint(nvg::RadialGradient(
			medalPos+medalBounds/2.0 +vec2(3,3), 
			(0.5 * medalSize)/2.0, (0.95 * medalSize)/2.0, vec4(0, 0, 0, 1), vec4(0, 0, 0, 0)));
		nvg::Fill();
		nvg::ClosePath();
	}

	nvg::BeginPath();
	nvg::FillPaint(nvg::TexturePattern(medalPos, medalBounds, 0, medal, 1.0));
	nvg::Rect(medalPos, medalBounds);
	nvg::Fill();
	nvg::ClosePath();
	
	//draw text
	nvg::BeginPath();
	nvg::FontFace(nvgFont);
	nvg::FontSize(fontSize);
	
	nvg::Translate(vec2(medalSize+3, textHeight + totalBounds.y/2.0 -textHeight/2.0));
	nvg::SkewX(skewAngle);
		
	if(shadows){
		nvg::FillColor(vec4(0, 0, 0, 1));
		nvg::FontBlur(3.0);
		nvg::Text(vec2(3, 3), str);
	}

	nvg::FillColor(vec4(1, 1, 1, 1));
	nvg::FontBlur(0.0);
	nvg::Text(vec2(0,0), str);
	
	nvg::ClosePath();
	nvg::ResetTransform();
}

void OnSettingsChanged() {
	if(showAuthor && showGold && showSilver && showBronze)
		UI::ShowNotification("Warning: Too many medals",
		"Displaying 4 medals conflicts with \"Next Medal\" message.",
		vec4(0.4, 0,0, 1),
		2000);
}
